! Day_Cli_Ann_v5.3.f90
! April 4, 2005
!
! Day_Cli_Ann_v5.1 Modifies FrequencyAnalysis_ver1b.f90
! The Output Combines Day_Cli_Ann_v4_1.f90 With FrequencyAnalysis_ver1b.f90
! Day_Cli_Ann_v5.1 Adds Batch File Processing April 11, 2005 
! Day_Cli_Ann_v5.2 Fixes Problem with Min#0 and Adds Missing Count
! Day_Cli_Ann_v5.3 Adds Counts of Single and Consecutive Days Above a Threshold
! December 2, 2005

! Uses ReadDaily.f90 for input and set-up
! File Names Should Include Data Type Abbreviation,
! That is "ppt," "snw," "qs," "tmax," or "tmin"
! Missing Data Code is -99
! 
! Traces (T) of Ppt or Snw are Set to 0
!
! In The Frequency Analysis Component:
! FrequencyAnalysis_ver1b Modifies and Improves Calulation of Percentiles
! FrequencyAnalysis_ver1b Uses Real Number Arrays
! Temperature Frequency Analysis Implemented 3/17/2005
! Zeros are Not Counted Except for TMAX and TMIN
! Thus, Only Days With Ppt, Snw, or Qw Are Used in Frequency Counts
! The Data Used Are From the Intervals Selected (i.e. Seasons)
! Thus, the Percentiles are Computed From the Sorted Seasonal Data
! The Annual Temperature Cycle is Removed From the Daily Temperature Data
!
! 
PROGRAM Day_Cli_Ann_v5_3
IMPLICIT NONE

CHARACTER :: DataIn*26,DateTmp*10, FileNameIn*120, FileNameOut*120, &
             & BeginDate*10,EndDate*10, First*10, Last*10, Tmp*4
CHARACTER :: Answer*1
CHARACTER :: BatchFileName*120, BatchFileIn*120,BatchFileOut*120, &
             & DirectoryIn*120,DirectoryOut*120, BatchAnswer*1 !Variables Reserved for Batch File
CHARACTER :: DataDate(57488)*10
 
  INTEGER :: OpenStatus, OpenStatus1, InputStatus,InputStatus1, NumberLines, &
              & AllocateStatus, BatchFileNumber 
  INTEGER :: I, J, K 
  INTEGER :: JulianDay1, JulianDay2, Month, Dayi, Yeari, Year1, Year2
  INTEGER :: DateIndex, NoIntervals, MissData, EndDay, PositionPpt, &
             & PositionSnw, PositionQw
     REAL :: Q1, Q2, Q3, P90, P95, P99, Min, Max, QTmp, Threshold
  INTEGER :: Days_GTQ3(1800), Days_GT90(1800),  Days_LTE90(1800), Single(1800), ConsecDay(1800) 
     REAL :: Avg(1800), AvgTempDev(1800), Ppt_GT95(1800), Ppt_GT90(1800), Ppt_LTE90(1800), &
	         & Ppt_GTE100(1800), Total(1800)
	 REAL :: Maximum(1800), Minimum(1800)
  INTEGER :: Days_GT95(1800), Days_GTE99(1800), Days_GTE100(1800), Missing(1800), ZeroDays(1800)
            
INTEGER, DIMENSION (1800,2) :: JulDayArray
    REAL, DIMENSION (57488) :: SortedArray 
    REAL, DIMENSION (:), ALLOCATABLE :: DayValue
    REAL, DIMENSION (:), ALLOCATABLE :: FreqArrayTmp
    REAL :: XPosition(1800)
    REAL :: DayValueTmp, PercentMissing, NumberDays, NAvg, NSort, Xplot 
!
! **********************************************************************************
! User Input
!
!
! Data Entry and File Names
!
1 WRITE (*,*) "Process a single file or multiple files ?"
  WRITE (*,*) "s = single file"
  WRITE (*,*) "m = multiple files"
	READ (*,*) BatchAnswer

IF (BatchAnswer=="s" .OR. BatchAnswer=="S") GOTO 5

! File Input for Batch Processing ! ***********************************************

9 WRITE (*,*) "Enter path and name of batch processing file"
		READ (*,*) BatchFileName
		!BatchFileName="c:\temp\BatchTest\BatchFileTest.txt"
WRITE (*,*) "Enter number of files to process"
		READ (*,*) BatchFileNumber

WRITE (*,*) "Enter drive letter and directory of input files"
		READ (*,*)DirectoryIn 
		!DirectoryIn="c:\temp\BatchTest"
WRITE (*,*) "Enter drive letter and directory of output files"
		READ (*,*)DirectoryOut
		!DirectoryOut="c:\Temp\BatchTest"
WRITE(*,*)

OPEN (UNIT=11,FILE=BatchFileName, STATUS="OLD", POSITION="REWIND", &
         ERR=9, IOSTAT=OpenStatus1)

Batch_Input_Loop:DO J=1,BatchFileNumber
!
     READ (UNIT=11, FMT=7,IOSTAT=InputStatus1) BatchFileIn,BatchFileOut,First,Last, &
      PercentMissing,Xplot   
		IF (InputStatus1 > 0) STOP "Input Error"
			IF (InputStatus1 < 0) EXIT !"End of File"
	7 FORMAT (A24, 1X, A24, 1X, A10, X, A10, X, F2.0, X, F4.0)
	 FileNameIn=TRIM(DirectoryIn)//"\"//BatchFileIn
	FileNameOut=TRIM(DirectoryOut)//"\"//BatchFileOut
	WRITE (*,*)"Processing: ", FileNameIn
	
      PositionPpt=INDEX (FileNameIn, "ppt")
      PositionSnw=INDEX (FileNameIn, "snw")
       PositionQw=INDEX (FileNameIn, "qw")

     CALL CountLines_Dates

     CALL SetUpData

! Calculate Temperature Statistics Before Detrending Daily Temperature
      IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) CALL TemperatureStatistics
      IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) CALL AnnualCycle
!
     CALL SetUpSort
!
     CALL sort(NSort, SortedArray)
!
     CALL Quartiles_Percentiles
!
      IF (PositionPpt>0 .OR. PositionSnw>0 .OR. PositionQw>0) THEN
	  CALL Ppt_Qw_Snw_FrequencyCount  
        ELSE IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) THEN 
 	  CALL TemperatureFrequencyCount
      END IF
!
! Open and Prepare to Write Output Files
OPEN (UNIT=10,FILE=FileNameOut, STATUS="NEW", ACTION="WRITE",POSITION="REWIND", &
IOSTAT=OpenStatus)
	IF (OpenStatus > 0) STOP "Cannot Open File"
!
     IF (PositionPpt>0 .OR. PositionSnw>0) CALL PptSnwOutput
     IF (PositionQw>0) CALL QwOutput  
     IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) CALL TemperatureOutput
!
CLOSE(UNIT=10)
!
DEALLOCATE (DayValue)
!
END DO Batch_Input_Loop ! **************************************************************************

CLOSE (UNIT=11)

GOTO 12
!
! Single File Data Entry ***************************************************************************
! 
5 WRITE (*,*) "Enter Name of Data File to Process"
		READ *, FileNameIn ! Data Set Processed with  NCDC.v3
!
PositionPpt=INDEX (FileNameIn, "ppt")! Check data type; if Position(ppt, snw, or qw)=0
									 !file is tmin or tmax, if Position>0 file
									 !is ppt, snw, or qw
PositionSnw=INDEX (FileNameIn, "snw")
!
!
PositionQw=INDEX (FileNameIn, "qw")
!	
WRITE (*,*) "Enter Output File Name"
		READ *, FileNameOut
		!FileNameOut="E:\Data\Projects_f90\FrequencyAnalysis\TestData\TestFreq.txt"
WRITE(*,*)
!
CALL CountLines_Dates
!
CALL DisplayFileInfo
!
WRITE (*,*)
WRITE (*,*) 'Enter beginning date of first season or year (i.e. 01/01/1953)'
		READ (*, FMT=10) First
		!First="01/01/1906"
WRITE (*,*) 'Enter ending date of first season or year (i.e. 12/31/1953)'
		READ (*, FMT=10) Last
		!Last="12/31/1906"
10 FORMAT(A10)
WRITE (*,*) "Enter acceptable number of missing values per interval, in percent"
  		READ *, PercentMissing
		!PercentMissing=20
WRITE (*,*) "Enter plotting position of first interval"
		READ *, XPlot
		!XPlot=1906
!
CALL SetUpData
!
! Calculate Temperature Statistics Before Detrending Daily Temperature
IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) CALL TemperatureStatistics
IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) CALL AnnualCycle
!
CALL SetUpSort
!
CALL sort(NSort, SortedArray)
!print *,  (SortedArray(I), I=1,NSort/2)
!
CALL Quartiles_Percentiles
!
IF (PositionPpt>0 .OR. PositionSnw>0 .OR. PositionQw>0) THEN
	CALL Ppt_Qw_Snw_FrequencyCount  
ELSE IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) THEN 
	CALL TemperatureFrequencyCount
END IF
!
! Open and Prepare to Write Output Files
OPEN (UNIT=10,FILE=FileNameOut, STATUS="NEW", ACTION="WRITE",POSITION="REWIND", &
IOSTAT=OpenStatus)
	IF (OpenStatus > 0) STOP "Cannot Open File"
!
  IF (PositionPpt>0 .OR. PositionSnw>0) CALL PptSnwOutput
  IF (PositionQw>0) CALL QwOutput  
  IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) CALL TemperatureOutput
!
CLOSE(UNIT=10)
!
DEALLOCATE (DayValue)
!
12	WRITE (*,*) "Process more data Y/N"
		READ (*,*) Answer
		IF (Answer=="Y".OR. Answer=="y") GOTO 1
		
	WRITE (*,*)
!
! *********************************************************************************
!
CONTAINS
!
! Count Lines and Find BeginDate and EndDate
!
SUBROUTINE CountLines_Dates
IMPLICIT NONE
!
OPEN (UNIT=10,FILE=FileNameIn, STATUS="OLD", ACTION="READ", &
	POSITION="REWIND",IOSTAT=OpenStatus)
	NumberLines=0
	   MissData=0
DO I=1, 54788 
	 READ(UNIT=10, FMT=15, IOSTAT=InputStatus) DataIn
	 15 FORMAT (A26)
		IF (Datain(12:14)=="-99") MissData=MissData+1
	    IF (I==1) BeginDate=DataIn(1:10)
	  	IF (InputStatus > 0) STOP "Input Error"
		IF (InputStatus < 0) EXIT !End of File
		EndDate=DataIn(1:10)
		NumberLines=NumberLines+1
END DO 
CLOSE(UNIT=10)
!
END SUBROUTINE CountLines_Dates
!
! Display File Information ********************************************************
!
SUBROUTINE DisplayFileInfo
IMPLICIT NONE
!
WRITE (*,*) "File is ", TRIM (FileNameIn)
WRITE (*,*)
WRITE (*,*) "     Number of days = ", NumberLines
WRITE (*,*) "     Missing Values = ", MissData
WRITE (*,*)
WRITE (*,*) "First date in file is ", BeginDate
WRITE (*,*)
WRITE (*,*) "         Last date is ", EndDate
WRITE (*,*)
!
END SUBROUTINE DisplayFileInfo ! ***************************************************
!
! Set-up Data for Analysis
!
SUBROUTINE	SetUpData ! ************************************************************
IMPLICIT NONE
!
!Find Julian Day of BeginDate and EndDate
 CALL JulDayFormat (BeginDate, Month, Dayi, Yeari)
 JulianDay1=julday(Month,Dayi,Yeari)
 CALL JulDayFormat (EndDate, Month, Dayi, Yeari) 
 JulianDay2=julday(Month,Dayi,Yeari)
!
! Allocate Array DayValue
!
  ALLOCATE (DayValue(JulianDay1:JulianDay2), STAT=AllocateStatus)
		IF (AllocateStatus/=0) STOP "Not Enough Memory"
!
! Read Data File in Form MM/DD/YYYY XXX and Setup Array DataValue
! Dimensioned by Julian Day
!
OPEN (UNIT=10,FILE=FileNameIn, STATUS="OLD", ACTION="READ", &
	POSITION="REWIND",IOSTAT=OpenStatus)
!
DateIndex=0
ReadFile:DO I=JulianDay1, JulianDay2
    DateIndex=DateIndex+1
	READ(UNIT=10, FMT=30, IOSTAT=InputStatus) DataIn
		IF (InputStatus > 0) STOP "Input Error"
			IF (InputStatus < 0) EXIT !End of File
30 FORMAT (A26)
!
! Read Data as Real
!
	IF (DataIn(14:26)=="T") DataIn(14:26)="0" !Change T(race) to 0
	READ (DataIn(12:26), *) DayValueTmp
!40 FORMAT(F12.2)
		        DayValue(I)=DayValueTmp
		            DateTmp=DataIn(1:10)
		DataDate(DateIndex)=DateTmp
!If (DataDate(DateIndex)(1:5)=="02/29") write (*,*) DataDate(DateIndex), DayValue(I)  			
END DO ReadFile
!
CLOSE (UNIT=10)  
!
!Build Array of Julian Dates for Analysis of Intervals
!
CALL JulDayFormat (EndDate, Month, Dayi, Yeari)
   EndDay=julday(Month,Dayi,Yeari)
NoIntervals=0  
DO I=1,1800
	CALL JulDayFormat (First, Month, Dayi, Yeari)
		JulDayArray(I,1)=julday(Month,Dayi,Yeari)
	CALL JulDayFormat (Last,Month, Dayi, Yeari)
		JulDayArray(I,2)=julday(Month,Dayi,Yeari)
	IF (JulDayArray(I,2) > EndDay) THEN 
		NoIntervals=I-1
		EXIT
	END IF
	!Increment Year in First and Last after Converting to Integer
	READ (First(7:10),*) Year1
		Year1=Year1+1
	WRITE (Tmp,'(I4)') Year1
		First(1:10)=First(1:6)//Tmp(1:4)
	READ (Last(7:10),*) Year2
		Year2=Year2+1
	WRITE (Tmp, '(I4)') Year2 
		 Last(1:10)=Last(1:6)//Tmp(1:4)
!
! write (*,*) JulDayArray(I,1), JulDayArray(I,2),& ! DEBUGS CORRECTLY
! JulDayArray(I,2)-JulDayArray(I,1)+1
END DO !CHECK FOR LEAP YEAR
! write (*,*) NoIntervals
!
END SUBROUTINE SetUpData ! **************************************************************
!
!
SUBROUTINE TemperatureStatistics ! ******************************************************
IMPLICIT NONE
!
!
! Calculate Temperature Statistics From Raw Data
DO I=1,NoIntervals
	NumberDays=JulDayArray(I,2)-JulDayArray(I,1)+1

	ALLOCATE (FreqArrayTmp(JulDayArray(I,1):JulDayArray(I,2)),STAT=AllocateStatus)
        IF (AllocateStatus/=0) STOP "Not Enough Memory"
	   FreqArrayTmp=DayValue(JulDayArray(I,1):JulDayArray(I,2))
	   	     Missing(I)=COUNT(FreqArrayTmp==-99)
	IF (Missing(I)/NumberDays*100<=PercentMissing) THEN				 
                     Total(I)=SUM(FreqArrayTmp, MASK=FreqArrayTmp .NE. -99)
				         NAvg=COUNT(FreqArrayTmp .NE. -99)
			           Avg(I)=Total(I)/NAvg
				   Minimum(I)=MINVAL(FreqArrayTmp, Mask=FreqArrayTmp .NE. -99)
				   Maximum(I)=MAXVAL(FreqArrayTmp, Mask=FreqArrayTmp .NE. -99)				 	     
	END IF
	IF (Missing(I)/NumberDays*100>PercentMissing) THEN
	                   Avg(I)=-99
	               Minimum(I)=-99
				   Maximum(I)=-99 
	END IF
        
	DEALLOCATE(FreqArrayTmp)
! write (*, *) XPosition(I)
! write (*, *) Days_GTQ3(I), Days_GT90(I), Days_GT95(I) 
END DO 

END SUBROUTINE TemperatureStatistics ! **************************************************
!
! 
SUBROUTINE AnnualCycle ! ****************************************************************
IMPLICIT NONE
!
! AnnualCycle.f90
! Annual Cycle Subroutine
! March 17, 2005
! Written for Use as a Subroutine Using Floating Point Numbers 
! This Program Computes the Daily Annual Cycle of Ppt, Snw, TMAX, TNMIN, and Qw Data
! Missing Values are Ignored. Leap Years are Used 
! The Data are Detrended by Removal of the Annual Cycle (Useful for Temperature Data)
! The Output Arrays are DayValue, Average, StdDev, Maximum, and Minimun for 366 Days
! Input Data is FileNameIn and NumberLines
! Generally, the Program Works for Data Beginning on 01/01/YYYY and Ending On 12/31/YYYY
! The Annual Cycle is Computed Using the Entire Time Series
!
         REAL :: Average(366), Maximum (366), Minimum(366), StdDev(366)
         REAL :: Index, NDays, Sum, Sum_of_Squares
      INTEGER :: Number_of_Days, Day, MthDays
CHARACTER (2) :: Day_Str, Month_Str
    CHARACTER :: Date_String(366)*5
!
!Create 366 Calendar Dates (01/01 to 12/31) ! *******************************************
!
Number_of_Days=0
DO J=1,12
		WRITE (Month_Str,'(I2)') J
			IF (J<10) Month_Str(1:1)="0"
	!Select number of days in month J
		Days_in_Month: SELECT CASE (J)
		CASE (2)
			MthDays=29
		CASE (4)
			MthDays=30
		CASE (6)
			MthDays=30
		CASE (9)
			MthDays=30
		CASE (11)
			MthDays=30
		CASE DEFAULT
			MthDays=31
		END SELECT Days_in_Month
			 DO Day=1, MthDays
			Number_of_Days=Number_of_Days+1
			WRITE (Day_Str,'(I2)') Day
			IF (Day<10) Day_Str(1:1)="0"
				Date_String(Number_of_Days)(1:5)=Month_Str//"/"//Day_Str				
			END DO
END DO ! ********************************************************************************
!
! Calculate Statistics for Each Calendar Date From 01/01 to 12/31 (Annual Cycle)
!
Minimum=100
Maximum=-200
DO I=1, 366
             Index=0
               Sum=0
	Sum_of_Squares=0
             NDays=0			 
	DO K=JulianDay1, JulianDay2
	       Index=Index+1
	   	   IF (Date_String(I)==DataDate(Index)(1:5) .AND. DayValue(K) .NE. -99) THEN   
	                    Sum=Sum+DayValue(K)
			 Sum_of_Squares=Sum_of_Squares+DayValue(K)**2
		              NDays=NDays+1
					  IF (DayValue(K)>Maximum(I)) Maximum(I)=DayValue(K)
					  IF (DayValue(K)<Minimum(I)) Minimum(I)=DayValue(K)
	           END IF
	END DO
  IF (NDays>=2) Average(I)=Sum/NDays !More Than 1 Year
  IF (NDays==1) Average(I)=Sum ! 1 year
  IF (NDays-1==0) StdDev(I)=-99
  IF ((Sum_of_Squares-(Sum**2/NDays))/NDays-1>=0) THEN
     StdDev(I)=SQRT((Sum_of_Squares-(Sum**2/NDays))/(NDays-1))
       ELSE
     StdDev(I)=0
  END IF
! write(*,*) Date_String(I), Average(I), StdDev(I), Maximum(I), Minimum(I) 
END DO ! ********************************************************************************
!
! Remove Annual Cycle From Array DayValue With Temperature Data ! ***********************
!
DO I=1, 366
    Index=0
    DO K=JulianDay1, JulianDay2
	       Index=Index+1
	   IF (Date_String(I)==DataDate(Index)(1:5) .AND. DayValue(K) .NE. -99) THEN
	       DayValue(K)=DayValue(K)-Average(I)
! write (*,*) DataDate(Index), Average(I), DayValue(K) !IF (Date_String(I)=="09/30")   
       END IF
   END DO
! write (*,*) K
END DO
!
END SUBROUTINE AnnualCycle ! **********************************************************
!
SUBROUTINE SetUpSort ! ****************************************************************
IMPLICIT NONE
!             
! Remove 0 and -99 From Ppt, Snw, and Qw Data
! Remove -99 From Temp Data
! Sorted Data is From Selected Intervals in JulDayArray (Seasons)
!
J=0
DO I=1, NoIntervals
   DO K=JulDayArray(I,1), JulDayArray(I,2)
        IF (PositionPpt>0 .OR. PositionSnw>0 .OR. PositionQw>0) THEN
		    IF (DayValue(K) .NE. -99 .AND. DayValue(K)>0) THEN
			    J=J+1
				SortedArray(J)=DayValue(K)
		    END IF
	    END IF
		IF (PositionPpt==0 .AND. PositionSnw==0 .AND. PositionQw==0) THEN
			IF (DayValue(K) .NE. -99) THEN
			    J=J+1
                SortedArray(J)=DayValue(K)
		    END IF
		END IF
	END DO
END DO
NSort=J
! write (*,*) NSort
END SUBROUTINE SetUpSort ! ***********************************************************
!
!
SUBROUTINE Quartiles_Percentiles ! ***************************************************
IMPLICIT NONE

! Empirical Distribution Function Method (Statistica)
! Percentile Calculation

Min=SortedArray(1)	
! Q1
QTmp=0.25*NSort
  IF (QTmp-INT(QTmp)==0) THEN
      Q1=SortedArray(QTmp)
  ELSE IF (QTmp-INT(QTmp)>0) THEN
      Q1=SortedArray(INT(QTmp)+1)
  END IF
! Q2
QTmp=0.5*NSort
  IF (QTmp-INT(QTmp)==0) THEN
      Q2=SortedArray(QTmp)
  ELSE IF (QTmp-INT(QTmp)>0) THEN
      Q2=SortedArray(INT(QTmp)+1)
  END IF
! Q3
QTmp=0.75*NSort
  IF (QTmp-INT(QTmp)==0) THEN
      Q3=SortedArray(QTmp)
  ELSE IF (QTmp-INT(QTmp)>0) THEN
      Q3=SortedArray(INT(QTmp)+1)
  END IF
! P90
QTmp=0.9*NSort
  IF (QTmp-INT(QTmp)==0) THEN
      P90=SortedArray(QTmp)
  ELSE IF (QTmp-INT(QTmp)>0) THEN
      P90=SortedArray(INT(QTmp)+1)
  END IF
! P95
QTmp=0.95*NSort
  IF (QTmp-INT(QTmp)==0) THEN
      P95=SortedArray(QTmp)
  ELSE IF (QTmp-INT(QTmp)>0) THEN
      P95=SortedArray(INT(QTmp)+1)
  END IF
! P99
QTmp=0.99*NSort
  IF (QTmp-INT(QTmp)==0) THEN
      P99=SortedArray(QTmp)
  ELSE IF (QTmp-INT(QTmp)>0) THEN
      P99=SortedArray(INT(QTmp)+1)
  END IF
!
Max=SortedArray(NSort)
! write (*,*) Min, Q1, Q2, Q3, P90, P95, P99, Max
END SUBROUTINE Quartiles_Percentiles ! *********************************************
!
!  
SUBROUTINE Ppt_Qw_Snw_FrequencyCount ! *********************************************
IMPLICIT NONE
!
DO I=1,NoIntervals
	NumberDays=JulDayArray(I,2)-JulDayArray(I,1)+1

	ALLOCATE (FreqArrayTmp(JulDayArray(I,1):JulDayArray(I,2)),STAT=AllocateStatus)
       IF (AllocateStatus/=0) STOP "Not Enough Memory"
	   FreqArrayTmp=DayValue(JulDayArray(I,1):JulDayArray(I,2))
	   	     Missing(I)=COUNT(FreqArrayTmp==-99)
	  IF (Missing(I)/NumberDays*100<=PercentMissing) THEN
			     Days_GT90(I)=COUNT(FreqArrayTmp>P90)
			      Ppt_GT90(I)=SUM(FreqArrayTmp,MASK=FreqArrayTmp>P90)
				Days_LTE90(I)=COUNT(FreqArrayTmp<=P90.AND. FreqArrayTmp &
				 .NE. -99 .AND. FreqArrayTmp .NE. 0)
				 Ppt_LTE90(I)=SUM(FreqArrayTmp, MASK=FreqArrayTmp<=P90 &
				  .AND. FreqArrayTmp .NE. -99 .AND. FreqArrayTmp .NE. 0)
				 Days_GT95(I)=COUNT(FreqArrayTmp>P95)
			    Days_GTE99(I)=COUNT(FreqArrayTmp>=P99)
				  Ppt_GT95(I)=SUM(FreqArrayTmp, MASK=FreqArrayTmp>P95)
			   Days_GTE100(I)=COUNT(FreqArrayTmp>=100)
			    Ppt_GTE100(I)=SUM(FreqArrayTmp, MASK=FreqArrayTmp>=100) !Not Snow
			         Total(I)=SUM(FreqArrayTmp, MASK=FreqArrayTmp .NE. -99)
				  ZeroDays(I)=COUNT(FreqArrayTmp==0)
				         NAvg=COUNT(FreqArrayTmp .NE. -99)
			           Avg(I)=Total(I)/NAvg
					   IF (Total(I) .NE. 0) THEN
			       Minimum(I)=MINVAL(FreqArrayTmp, MASK=FreqArrayTmp .NE. -99 &
				     .AND. FreqArrayTmp .NE. 0)
					   ELSE
					     Minimum(I)=-99
					   END IF
				   Maximum(I)=MAXVAL(FreqArrayTmp, MASK=FreqArrayTmp .NE. -99)
		CALL ConsecutiveDays
    END IF
	IF (Missing(I)/NumberDays*100>PercentMissing) THEN
	                   Avg(I)=-99
                 Days_GT90(I)=-99
			      Ppt_GT90(I)=-99
				Days_LTE90(I)=-99
				 Ppt_LTE90(I)=-99
				 Days_GT95(I)=-99
                Days_GTE99(I)=-99
				  Ppt_GT95(I)=-99
			   Days_GTE100(I)=-99
                Ppt_GTE100(I)=-99
                     Total(I)=-99
				  ZeroDays(I)=-99
			  	   Minimum(I)=-99
				   Maximum(I)=-99
			     ConsecDay(I)=-99
			        Single(I)=-99
	END IF 
        XPosition(I)=XPlot
	           XPlot=XPlot+1
! write (*, *) XPosition(I)
! write (*, *) Days_GT90(I), Ppt_GT90(I), Days_LTE90(I), Ppt_LTE90(I),&
        ! & Days_GT95(I), Ppt_GT95(I)
! write (*, *)  Days_GTE100(I), Ppt_GTE100(I)
! write (*, *) Days_GTE99 !NAvg, Avg(I), Days_GT90(I), 
DEALLOCATE(FreqArrayTmp)  
END DO 
!		
END SUBROUTINE Ppt_Qw_Snw_FrequencyCount ! ***************************************
!
!
SUBROUTINE TemperatureFrequencyCount ! *******************************************
IMPLICIT NONE
!
DO I=1,NoIntervals
	NumberDays=JulDayArray(I,2)-JulDayArray(I,1)+1

	ALLOCATE (FreqArrayTmp(JulDayArray(I,1):JulDayArray(I,2)),STAT=AllocateStatus)
        IF (AllocateStatus/=0) STOP "Not Enough Memory"
	   FreqArrayTmp=DayValue(JulDayArray(I,1):JulDayArray(I,2))
	   	     Missing(I)=COUNT(FreqArrayTmp==-99)
	IF (Missing(I)/NumberDays*100<=PercentMissing) THEN
				 Days_GTQ3(I)=COUNT(FreqArrayTmp>Q3)
			     Days_GT90(I)=COUNT(FreqArrayTmp>P90)
		    	 Days_GT95(I)=COUNT(FreqArrayTmp>P95)
                     Total(I)=SUM(FreqArrayTmp, MASK=FreqArrayTmp .NE. -99)
				         NAvg=COUNT(FreqArrayTmp .NE. -99)
			    AvgTempDev(I)=Total(I)/NAvg
		CALL ConsecutiveDays				 	     
	END IF
	IF (Missing(I)/NumberDays*100>PercentMissing) THEN
	             AvgTempDev(I)=-99
	              Days_GTQ3(I)=-99
                  Days_GT90(I)=-99
			      Days_GT95(I)=-99
				  ConsecDay(I)=-99
				     Single(I)=-99
				  		 
	END IF
        XPosition(I)=XPlot
		       XPlot=XPlot+1
	DEALLOCATE(FreqArrayTmp)
! write (*, *) XPosition(I)
! write (*, *) Days_GTQ3(I), Days_GT90(I), Days_GT95(I) 
END DO 
!		
END SUBROUTINE TemperatureFrequencyCount ! *************************************
!
!
!
SUBROUTINE PptSnwOutput ! ******************************************************
IMPLICIT NONE
!
WRITE (Unit=10, FMT=50) "Sample interval is ", First, Last
50  FORMAT(X, A19, A5, X, A5)
WRITE (UNIT=10, FMT=60) "Acceptable missing, in percent", PercentMissing, "Threshold = ", Threshold
60  FORMAT(X, A30, F5.1, 5X, A12, F5.1)
!
	WRITE(UNIT=10, FMT=70) "Statistics"
	WRITE(Unit=10, FMT=80) "Min = ", Min	
	WRITE(Unit=10, FMT=80) "25% = ", Q1
	WRITE(Unit=10, FMT=80) "50% = ", Q2
	WRITE(Unit=10, FMT=80) "75% = ", Q3
	WRITE(Unit=10, FMT=80) "90% = ", P90
	WRITE(Unit=10, FMT=80) "95% = ", P95
	WRITE(Unit=10, FMT=80) "Max = ", Max
70 FORMAT(X, A10)
80 FORMAT(X, A6, F12.2)
!
 IF (PositionSnw>0) GOTO 300
WRITE(Unit=10, FMT=100) "Year", "TotPpt", "Min#0","Max", "Days>90", "Ppt>90", &
     & "Days<=90", "Ppt<=90", "Days>95", "Ppt>95", "Days>=1in", &
     & "Ppt>=1in", "ZeroDays", "Missing", "Single>4", "Consec>4"

100  FORMAT (2X, A4, 4X, A6, 4X, A5, 4X, A3, 3X, A7, 2X, A6, X, A8, X, &
      & A7, X, A7, 2X, A6, 2X, A9, X, A9, X, A8, X, A7, X, A8, X, A8)

!
DO I=1, NoIntervals
	WRITE(Unit=10, FMT=110) XPosition(I), Total(I), Minimum(I), Maximum(I), & 
	Days_GT90(I), Ppt_GT90(I), Days_LTE90(I), Ppt_LTE90(I), Days_GT95(I), Ppt_GT95(I), &
	& Days_GTE100(I), Ppt_GTE100(I),  ZeroDays(I), Missing(I), Single(I), ConsecDay(I)
END DO
!
110 FORMAT(X, F6.1, 2X, F7.1, 2X, F7.1, 2X, F7.1, 2X, I4, 2X, F7.1, 3X, I4, 4X, F7.1, &
    & 2X, I4, 4X, F7.1, 3X, I4, 5X, F7.1, 4X, I4, 4X, I4, 4X, I4, 5X, I4)
!
GOTO 400
!
300 WRITE(Unit=10, FMT=120) "Year", "TotSnw", "Min#0", "Max", "Days>90", "Snw>90", "Days<=90", &
    & "Snw<=90", "Days>95", "Snw>95", "ZeroDays", "Missing", "Single>Q1", "Consec>Q1"
120  FORMAT(3X, A4, 4X, A6, 4X, A5, 4X, A3, 3X, A7, X, A6, 2X, A8, 2X, A7, X, &
     & A7, 2X, A6, X, A8, X, A7, X, A9, X, A9)
DO I=1, NoIntervals
	WRITE(Unit=10, FMT=130) XPosition(I), Total(I), Minimum(I), Maximum(I), Days_GT90(I), &
	& Ppt_GT90(I), Days_LTE90(I), Ppt_LTE90(I), Days_GT95(I), Ppt_GT95(I), &
	& ZeroDays(I), Missing(I), Single(I), ConsecDay(I)
END DO
130 FORMAT(X, F7.1, 2X, F7.1, 2X, F7.1, 2X,F7.1, 2X, I4, 2X, F7.1, 3X, I4, 4X, &
    & F7.1, 2X, I4, 4X, F7.1, 2X, I4, 4X, I4, 3X, I4, 6X, I4)
!
400 END SUBROUTINE PptSnwOutput ! **********************************************
!
!
SUBROUTINE QwOutput ! **********************************************************
IMPLICIT NONE
!
WRITE (Unit=10, FMT=50) "Sample interval is ", First, Last
50  FORMAT(X, A19, A5, X, A5)
WRITE (UNIT=10, FMT=60) "Acceptable missing, in percent", PercentMissing, "Threshold = ", Threshold
60  FORMAT(X, A30, F5.1, 5X, A12, F12.2)
!
	WRITE(UNIT=10, FMT=70) "Statistics"
	WRITE(Unit=10, FMT=80) "Min = ", Min	
	WRITE(Unit=10, FMT=80) "25% = ", Q1
	WRITE(Unit=10, FMT=80) "50% = ", Q2
	WRITE(Unit=10, FMT=80) "75% = ", Q3
	WRITE(Unit=10, FMT=80) "90% = ", P90
	WRITE(Unit=10, FMT=80) "95% = ", P95
    WRITE(Unit=10, FMT=80) "99% = ", P99
	WRITE(Unit=10, FMT=80) "Max = ", Max
70 FORMAT(X, A10)
80 FORMAT(X, A6, F12.2)
!
WRITE(Unit=10, FMT=140) "Year", "AvgQ", "Min#0", "Max", "Days>90", "Days<=90", "Days>95", &
      & "Days>=99", "ZeroDays", "Missing", "Single>95", "Consec>95"
140 FORMAT(2X, A4, 8X, A4, 5X, A5, 5X, A3, 4X, A7, X, A8, X, A7, X, A8, X, A8, X, A7, X, A9, X, A9)
!
DO I=1, NoIntervals
	WRITE(Unit=10, FMT=150) XPosition(I), Avg(I), Minimum(I), Maximum(I), Days_GT90(I), &
	& Days_LTE90(I), Days_GT95(I), Days_GTE99(I), ZeroDays(I), Missing(I), Single(I), ConsecDay(I)
END DO
150  FORMAT(X, F6.1, 2X, F10.3, 2X, F8.2, 2X, F8.1, 2X, I4, 4X, I4, &
      & 4X, I4, 5X, I4, 5X, I4, 4X, I4, 5X, I4, 6X, I4)
END SUBROUTINE QwOutput ! ****************************************************
!
!
SUBROUTINE TemperatureOutput ! ***********************************************
IMPLICIT NONE
!
WRITE (Unit=10, FMT=50) "Sample interval is ", First, Last
50  FORMAT(X, A19, A5, X, A5)
WRITE (UNIT=10, FMT=60) "Acceptable missing, in percent", PercentMissing, "Threshold(Dev) = ", Threshold
60  FORMAT(X A30, F5.1, 5X, A17, F12.2)
!
WRITE(UNIT=10, FMT=70), "Statistics (Deviation)"
    WRITE(UNIT=10, FMT=80), "Min = ",Min
	WRITE(Unit=10, FMT=80), "Q1 = ", Q1
	WRITE(Unit=10, FMT=80), "Q2 = ", Q2
	WRITE(Unit=10, FMT=80), "Q3 = ", Q3
	WRITE(Unit=10, FMT=80), "90% = ", P90
	WRITE(Unit=10, FMT=80), "95% = ", P95
	WRITE(UNIT=10, FMT=80), "Max = ", Max
70 FORMAT(X, A22)
80 FORMAT(X, A5, F12.2)
!
WRITE(Unit=10, FMT=120) "Year", "Avg", "Min", "Max", "Days>Q3", "Days>90", "Days>95", &
& "AvgDev", "Missing", "Single>90", "Consec>90"
120  FORMAT(2X, A4, 7X, A3, 7X, A3, 6X, A3, 4X, A7, X, A7, X, A7, 3X, A6, 3X, A7, X, A9, X, A9)
!
DO I=1, NoIntervals
WRITE(Unit=10, FMT=130), XPosition(I), Avg(I), Minimum(I), Maximum(I), Days_GTQ3(I), & 
& Days_GT90(I), Days_GT95(I), AvgTempDev(I), Missing(I), Single(I), ConsecDay(I)
END DO
130 FORMAT(X, F6.1, 4X, F8.4, 3X, F6.2, 3X, F6.2, 3X, I4, 3X, I4, 5X, I4, 4X, F8.4, 3X, I4, 5X, I4, 6X, I4)
!
END SUBROUTINE TemperatureOutput ! ******************************************
!
!
SUBROUTINE JulDayFormat(Date_Str, Mth, Day, Year) ! *************************
IMPLICIT NONE
! 
CHARACTER, INTENT (IN) :: Date_Str*10
INTEGER, INTENT (OUT) :: Mth, Day, Year 
	READ (Date_Str(1:2),*) Mth
	READ (Date_Str(4:5),*) Day
	READ (Date_Str(7:10),*) Year
!
END SUBROUTINE JulDayFormat ! ***********************************************
!
!Julian Day from Calendar Date.  Code from Press and others (1992, p.13).
!Modifications suggested by Ahlquist (1999) were made 1/13/2005
! 
FUNCTION julday(mm,id,iyyy) ! ***********************************************
IMPLICIT NONE
      INTEGER julday,id,iyyy,mm,IGREG
      PARAMETER (IGREG=15+31*(10+12*1582))
      INTEGER ja,jm,jy
      jy=iyyy
      if (jy.eq.0) pause 'julday: there is no year zero'
      if (jy.lt.0) jy=jy+1
      if (mm.gt.2) then
        jm=mm+1
      else
        jy=jy-1
        jm=mm+13
      endif
      julday=int(365.25*jy)+int(30.6001*jm)+id+1720995
      if (id+31*(mm+12*iyyy).ge.IGREG) then
        ja=jy/100 !ja=int(0.01*jy) Original line
        julday=julday+2-ja+int(0.25*ja)
      endif

      END FUNCTION julday !  **********************************************
!
!Quicksort Routine
!A Quicksort routine from Press and others (1992, p. 323-325)
!
SUBROUTINE sort(n,arr)! ***************************************************
IMPLICIT NONE
    
         REAL, INTENT(IN) :: n
         REAL, INTENT(OUT)::arr(n) 
		 INTEGER M,NSTACK					 
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL a,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          do 11 i=j-1,l,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
11        continue
          i=l-1
2         arr(i+1)=a
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l).gt.arr(l+1))then
          temp=arr(l)
          arr(l)=arr(l+1)
          arr(l+1)=temp
        endif
        i=l+1
        j=ir
        a=arr(l+1)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        goto 3
5       arr(l+1)=arr(j)
        arr(j)=a
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
END SUBROUTINE sort !****************************************************
!
!
SUBROUTINE ConsecutiveDays ! ***************************************************
!
IMPLICIT NONE
   
INTEGER :: J, K, SingleDay, Consec, N, IA(366) ! IA is the Array of Julian Days
!
!Set Threshold for Data Type
IF (PositionPpt>0) THEN
       Threshold=4 ! Threshold=1.02 mm
    ELSE IF (PositionSnw>0) THEN
       Threshold=Q1
    ELSE IF (PositionQw>0) THEN
        Threshold=P95
    ELSE
        Threshold=P90 ! Temperature Data
END IF 
!
K=0
!
! Find Index of Daily Values Exceeding Threshold
DO J=JulDayArray(I, 1), JulDayArray(I, 2)
!
   IF (FreqArrayTmp(J)==-99) CYCLE
   IF (FreqArrayTmp(J)>Threshold) THEN
            K=K+1
	IA(K)=J
	!write(*,*) IA(K)
   END IF
!
END DO  
!
SingleDay=0
   ConSec=0
        N=K
!
IF (K==0) GOTO 600 ! No Values Larger Than Threshold 
!write (*,*) N
!
DO J=1, N-1
!
    !Single Day Occurrence
!
    IF (J==1 .AND. IA(J+1)-IA(J)>1) SingleDay=SingleDay+1 !First Day
    IF (J>1 .AND. IA(J)-IA(J-1)>1 .AND. IA(J+1)-IA(J)>1) SingleDay=SingleDay+1
    
    !Consecutive Day Occurrence
!
    IF (IA(J+1)-IA(J)==1) ConSec=ConSec+1 
    IF (J>1 .AND. IA(J)-IA(J-1)==1 .AND. IA(J+1)-IA(J)>1) ConSec=ConSec+1
!
END DO
!
    IF (IA(N)-IA(N-1)>1) SingleDay=SingleDay+1 !Last Day
    IF (IA(N)-IA(N-1)==1) ConSec=ConSec+1      !Last Consecutive Day
!
600    Single(I)=SingleDay
    ConsecDay(I)=ConSec
!
!WRITE(*,*) XPlot, Single(I), ConsecDay(I)
!stop
!
END SUBROUTINE ConsecutiveDays ! **********************************************!
!
!
END PROGRAM Day_Cli_Ann_v5_3