! Month_from_Day_v2.1
! Program calculates monthly statistics of precipitation, snow, 
!    temperature, and discharge from daily data
! July 15, 1998
! July 14, 2004
! Modified to output up to 99.99 inches per month
! January 14, 2005
! Modified version 1.1 for leap year exceptions at century
! April 12, 2005 Modified to Accept Real Numbers
! Limitations: Data Must Begin On the First Day of Any Month
!    File Name Must Include Data Type Suffix
!    That is ppt, snw, tmax, tmin, or qw
! April 6, 2006 Fixed Temperature File Name Conventions

PROGRAM Month_v2_1

IMPLICIT NONE

CHARACTER :: BatchFileName*120, BatchFileIn*120,BatchFileOut*120, &
             & DirectoryIn*120,DirectoryOut*120, BatchAnswer*1 !Variables Reserved for Batch File
CHARACTER :: Answer*1,FileNameIn*120, FileNameOut*120, BeginDate*10,EndDate*10
CHARACTER :: DataIn*26, DateTmp*10, First*10, Last*10
CHARACTER :: DataDate(57488)*10
  INTEGER ::  OpenStatus1, InputStatus1, BatchFileNumber
  INTEGER :: AllocateStatus, I,J,InputStatus, NumberLines, OpenStatus
  INTEGER :: PositionQw, PositionPpt, PositionSnw, PositionMint, PositionMaxt 
  INTEGER :: DateIndex, DayIndex, Day_of_Month, Mth, YearCount, YearNum
  INTEGER :: FirstYear, LastYear, MonthNumber, Days_in_Month, M

REAL, ALLOCATABLE, DIMENSION(:) :: DayValue
REAL :: TotalPpt_or_Temp, Missing, MissData, PercentMissing, DayValueTmp, Xplot
REAL,DIMENSION(1800,13) :: Ppt_or_Temp
!
!Data Entry and File Names
!
! User Input

! Data Entry and File Names
!
1 WRITE (*,*) "Process a single file or multiple files ?"
  WRITE (*,*) "s = single file"
  WRITE (*,*) "m = multiple files"
	READ (*,*) BatchAnswer

IF (BatchAnswer=="s" .OR. BatchAnswer=="S") GOTO 10

! File Input for Batch Processing ! ***********************************************

9 WRITE (*,*) "Enter path and name of batch processing file"
		READ (*,*) BatchFileName
		WRITE(*,*)
WRITE (*,*) "Enter number of files to process"
		READ (*,*) BatchFileNumber
        WRITE(*,*)
WRITE (*,*) "Enter drive letter and directory of input files"
		READ (*,*)DirectoryIn 
		WRITE(*,*)
WRITE (*,*) "Enter drive letter and directory of output files"
		READ (*,*)DirectoryOut
		WRITE(*,*)

OPEN (UNIT=11,FILE=BatchFileName, STATUS="OLD", POSITION="REWIND", &
         ERR=9, IOSTAT=OpenStatus1)

Batch_Input_Loop:DO J=1,BatchFileNumber
!
     READ (UNIT=11, FMT=7,IOSTAT=InputStatus1) BatchFileIn,BatchFileOut,First,Last, &
      PercentMissing,Xplot   
		IF (InputStatus1 > 0) STOP "Input Error"
			IF (InputStatus1 < 0) EXIT !"End of File"
	7 FORMAT (A24, 1X, A24, 1X, A10, X, A10, X, F2.0, X, F4.0)
	 FileNameIn=TRIM(DirectoryIn)//"\"//BatchFileIn
	FileNameOut=TRIM(DirectoryOut)//"\"//BatchFileOut
        WRITE (*,*)"Processing: ", FileNameIn
     
	CALL CountLines_Dates

	CALL SetUpData

	READ (BeginDate(7:10),FMT=30) FirstYear
	READ (EndDate(7:10),FMT=30) LastYear
		30 FORMAT (I4)

        PositionPpt=INDEX (FileNameIn, "ppt")
        PositionSnw=INDEX (FileNameIn, "snw")
         PositionQw=INDEX (FileNameIn, "qw")
       PositionMint=INDEX(FileNameIn, "tmin")
       PositionMaxt=INDEX(FileNameIn, "tmax")	

       CALL DayIndexCalculations

  IF (PositionPpt>0 .OR. PositionSnw> 0) CALL Array_1_Output
IF (PositionMint>0 .OR. PositionMaxt> 0) CALL Array_1_Output
                       IF (PositionQw>0) CALL ArrayQwOutput

!    
CLOSE(UNIT=10)
!
DEALLOCATE (DayValue)
!
END DO Batch_Input_Loop ! **************************************************************************

CLOSE (UNIT=11)

GOTO 12
!
! Single File Data Entry ***************************************************************************
10 WRITE(*,*)
   WRITE (*,*) "Enter Name of Data File to Process"
  	 READ (*,*) FileNameIn
   WRITE(*,*)
!
OPEN (UNIT=10,FILE=FileNameIn, STATUS="OLD", ACTION="READ", &
	POSITION="REWIND",IOSTAT=OpenStatus, ERR=10)

WRITE (*,*) "Enter Output File Name"
	READ (*,*) FileNameOut
WRITE (*,*)

WRITE (*,*) "Enter Acceptable Number of Missing Values per Month, in Percent"
	READ (*,*) PercentMissing

WRITE (*,*) "Reading File and Processing"

CALL CountLines_Dates

WRITE (*,*)
WRITE (*,*) "Input File is ", FileNameIn
WRITE (*,*) "Data begins ", BeginDate
WRITE (*,*) "Data ends   ", EndDate
WRITE (*,*)

CALL SetUpData

READ (BeginDate(7:10),FMT=35) FirstYear
READ (EndDate(7:10),FMT=35) LastYear
35 FORMAT (I4)

        PositionPpt=INDEX (FileNameIn, "ppt")
        PositionSnw=INDEX (FileNameIn, "snw")
         PositionQw=INDEX (FileNameIn, "qw")
       PositionMint=INDEX(FileNameIn, "tmin")
       PositionMaxt=INDEX(FileNameIn, "tmax")	

CALL DayIndexCalculations

  IF (PositionPpt>0 .OR. PositionSnw> 0) CALL Array_1_Output
IF (PositionMint>0 .OR. PositionMaxt> 0) CALL Array_1_Output
                       IF (PositionQw>0) CALL ArrayQwOutput
DEALLOCATE (DayValue)

WRITE (*,*)
!
12	WRITE (*,*) "Process more data Y/N"
		READ (*,*) Answer
		IF (Answer=="Y".OR. Answer=="y") GOTO 1
		
	WRITE (*,*)
!
! *********************************************************************************
!
CONTAINS ! *********************************************************************
!
! Count Lines and Find BeginDate and EndDate
!
SUBROUTINE CountLines_Dates
IMPLICIT NONE
!
OPEN (UNIT=10,FILE=FileNameIn, STATUS="OLD", ACTION="READ", &
	POSITION="REWIND",IOSTAT=OpenStatus)
	NumberLines=0
	   MissData=0
DO I=1, 54788 
	 READ(UNIT=10, FMT=15, IOSTAT=InputStatus) DataIn
	 15 FORMAT (A26)
		IF (Datain(12:14)=="-99") MissData=MissData+1
	    IF (I==1) BeginDate=DataIn(1:10)
	  	IF (InputStatus > 0) STOP "Input Error"
		IF (InputStatus < 0) EXIT !End of File
		EndDate=DataIn(1:10)
		NumberLines=NumberLines+1
END DO 
CLOSE(UNIT=10)
!
END SUBROUTINE CountLines_Dates
!
! Set-up Data for Analysis
!
SUBROUTINE SetUpData ! ************************************************************
IMPLICIT NONE
!
! Allocate Array DayValue
!
  ALLOCATE (DayValue(NumberLines), STAT=AllocateStatus) 
		IF (AllocateStatus/=0) STOP "Not Enough Memory"
!
! Read Data File in Form MM/DD/YYYY XXX and Setup Array DataValue
!
OPEN (UNIT=10,FILE=FileNameIn, STATUS="OLD", ACTION="READ", &
	POSITION="REWIND",IOSTAT=OpenStatus)
!
DateIndex=0
ReadFile:DO I=1, NumberLines
    DateIndex=DateIndex+1
	READ(UNIT=10, FMT=30, IOSTAT=InputStatus) DataIn
		IF (InputStatus > 0) STOP "Input Error"
			IF (InputStatus < 0) EXIT !End of File
30 FORMAT (A26)
!
! Read Data as Real
!
	IF (DataIn(14:26)=="T") DataIn(14:26)="0" !Change T(race) to 0
	READ (DataIn(12:26), *) DayValueTmp
!40 FORMAT(F12.2)
		        DayValue(I)=DayValueTmp
		            DateTmp=DataIn(1:10)
		DataDate(DateIndex)=DateTmp
!If (DataDate(DateIndex)(1:5)=="02/29") write (*,*) DataDate(DateIndex), DayValue(I)  			
END DO ReadFile
!
CLOSE (UNIT=10)

END SUBROUTINE SetUpData ! ***************************************************************
!
!
! ****************************************************************************************
!
SUBROUTINE DayIndexCalculations

READ (BeginDate(1:2),FMT=300) MonthNumber !Initialize MonthNumber from first date in file
300 FORMAT (I2) 
          M=1	!Dummy variable to set array index
  YearCount=0
Ppt_or_Temp=-99

DO YearNum=FirstYear, LastYear
	YearCount=YearCount+1
	Ppt_or_Temp(YearCount,1)=YearNum
	DO Mth=MonthNumber, 12

			CALL Find_Days_in_Month
			      Day_of_Month=0
			  TotalPpt_or_Temp=0
			           Missing=0
		DO DayIndex=M,M+Days_in_Month-1 
				Day_of_Month=Day_of_Month+1
				IF (DayValue(DayIndex)/=-99)THEN
					TotalPpt_or_Temp=TotalPpt_or_Temp+DayValue(DayIndex)
				ELSE
					Missing=Missing+1
				END IF
 !WRITE (*,*) YearNum, Mth, YearCount, Day_of_Month, DayIndex
		END DO
!IF (TotalPpt_or_Temp>650) WRITE(*,*) TotalPpt_or_Temp,YearNum,YearCount, Mth
			IF (Missing/Days_in_Month*100>PercentMissing) THEN
				Ppt_or_Temp(YearCount,Mth+1)=-99 
			ELSE IF (PositionPpt>0 .OR. PositionSnw>0) THEN
				Ppt_or_Temp(YearCount,Mth+1)=TotalPpt_or_Temp
			ELSE IF (PositionPpt==0 .OR. PositionSnw==0) THEN
				Ppt_or_Temp(YearCount,Mth+1)=TotalPpt_or_Temp/(Days_in_Month-Missing)
			END IF				
		M=DayIndex
		IF (DayIndex>NumberLines) EXIT ! Needed To Correct Array Bounds With Water Year Files

 	END DO
	  MonthNumber=1
END DO

END SUBROUTINE	DayIndexCalculations ! ****************************************************
!
!
SUBROUTINE Find_Days_in_Month ! ***********************************************************
	SELECT CASE (Mth)
		CASE (2)
			Days_in_Month=28
			IF (MOD(YearNum,4)==0 .AND. MOD(YearNum,100).NE.0) Days_in_Month=29  
			IF (MOD(YearNum, 400)==0) Days_in_Month=29 
		CASE (4)
			Days_in_Month=30
		CASE (6)
			Days_in_Month=30
		CASE (9)
			Days_in_Month=30
		CASE (11)
			Days_in_Month=30
		CASE DEFAULT
			Days_in_Month=31
   END SELECT
END SUBROUTINE Find_Days_in_Month ! *****************************************************
!
!
SUBROUTINE Array_1_Output ! *************************************************************

OPEN (UNIT=10,FILE=FileNameOut, STATUS="NEW", ACTION="WRITE", POSITION="REWIND", &
      IOSTAT=OpenStatus)
	  IF (OpenStatus > 0) STOP "Cannot Open New File"
 
WRITE (UNIT=10,FMT=400) "Acceptable missing values per month, in percent", PercentMissing
400 FORMAT (1X, A51, F5.1)
WRITE (UNIT=10,FMT=500) "Year", "Jan", "Feb","Mar","Apr","May","Jun","Jul","Aug","Sep",&
						"Oct","Nov","Dec"
500 FORMAT (2X, A4, 6X, A3, 7X, A3, 7X, A3, 7X, A3, 7X, A3, 7X, A3, 7X, A3, 7X, A3, 7X, &
           & A3, 7X, A3, 7X, A3, 7X, A3)

DO I=1,YearCount
	   
WRITE(UNIT=10,FMT=600) (Ppt_or_Temp(I,J),J=1,13)
600 FORMAT (1X, F6.1, 2X, F8.2, 2X, F8.2, 2X, F8.2, 2X, F8.2, 2X, F8.2, 2X, F8.2, 2X, &
            &F8.2, 2X, F8.2, 2X, F8.2, 2X, F8.2, 2X, F8.2, 2X, F8.2)

END DO

CLOSE (UNIT=10)

END SUBROUTINE Array_1_Output ! **********************************************************
!
!
SUBROUTINE ArrayQwOutput
 
OPEN (UNIT=10,FILE=FileNameOut, STATUS="NEW", ACTION="WRITE", POSITION="REWIND", &
      IOSTAT=OpenStatus)
	  IF (OpenStatus > 0) STOP "Cannot Open File"
WRITE (UNIT=10,FMT=400) "Acceptable missing values per month, in percent", PercentMissing
400 FORMAT (1X, A51, F5.1)
WRITE (UNIT=10,FMT=500) "Year", "Jan", "Feb","Mar","Apr","May","Jun","Jul","Aug","Sep",&
						"Oct","Nov","Dec"
500 FORMAT (2X, A4, 6X, A3, 8X, A3, 8X, A3, 8X, A3, 8X, A3, 8X, A3, &
      & 8X, A3, 8X, A3, 8X, A3, 8X, A3, 8X, A3, 8X, A3)

DO I=1,YearCount
	   
WRITE(UNIT=10,FMT=600) (Ppt_or_Temp(I,J),J=1,13)
600 FORMAT (1X, F6.1, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, &
           & F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, F9.2, 2X, &
		   & F9.2, 2X, F9.2, 2X, F9.2)

END DO

CLOSE (UNIT=10)

END SUBROUTINE ArrayQwOutput ! ************************************************************* 

END PROGRAM Month_v2_1
