! SetUpDailyTable.f90
! September 14, 2005
! Program Builds a Table of Daily Values
! Program Should Function with ppt, snw, qw, tmin, and tmax Input Files
! Input Files are Those Created with NCDC_LFv4
! File Names are Read From Columns 1-25 of the Batch File
! Output File is Data (Row) by Station (Column)
! Trace (T) Amounts of Precipitation are Set to Zero

! *****************************************************************************************
PROGRAM SetUpDailyTable
IMPLICIT NONE
! *****************************************************************************************
CHARACTER :: FileNameOut*120
CHARACTER :: FileNameIn (200)*120
CHARACTER :: DataType*4, Temp*200, First*10, Last*10
CHARACTER :: BatchFileName*120, BatchFileIn*120, &
			 & DirectoryIn*120, DirectoryOut*120
CHARACTER :: Cal_Date*10, Month_str*2, Day_str*2, DateTemp*10, Year_str*4, Input_str*10, &
			 & Answer*1, Response_str*1
  INTEGER :: JulianDay, Mth, Day, Year
  INTEGER :: DataColumn, OpenStatus, OpenStatus1, InputStatus,InputStatus1, & 
				& NumberLines, BatchFileNumber 
  INTEGER :: I, J, K
  INTEGER :: JulianDay1, JulianDay2, Month, Dayi, Yeari, Year1, Year2
  INTEGER :: PositionTxt, FirstJulianDay, LastJulianDay
  
     REAL :: DayValue(54788), DayValueTemp  
  
CHARACTER, DIMENSION (:), ALLOCATABLE :: DataDate*10 ! Array of Calendar Dates     
     REAL, DIMENSION (:,:), ALLOCATABLE :: OutputTable 
!
! User Input ********************************************************************************
!
! File Input for Batch Processing 
!
1 WRITE (*,*) "Enter path and name of batch processing file"
				READ (*,*) BatchFileName
				WRITE(*,*)
WRITE (*,*) "Enter drive letter and directory of input files"
				READ (*,*) DirectoryIn 
				WRITE(*,*)
WRITE (*,*) "Enter drive letter and directory of output file"
				READ (*,*) DirectoryOut
				WRITE(*,*)
WRITE (*,*) "Enter name of output table"
				READ (*,*) FileNameOut
					FileNameOut=TRIM(DirectoryOut)//"\"//FileNameOut
				WRITE (*,*)
WRITE (*, *) "Enter first date of output table (i.e., 01/01/1900)"
				READ (*, FMT=9) First
				WRITE (*,*)			
WRITE (*, *) "Enter last date of output table (i.e., 12/31/2004)"
				READ (*, FMT=9) Last		
				WRITE (*,*)
9 FORMAT(A10) 
!!
CALL CountFiles
!
CALL SetUpOutputTable
!
FirstJulianDay=JulianDay1 ! First and Last Date of Output Table
 LastJulianDay=JulianDay2
!
CALL ReadBatchfile
!
CALL ReadInputFile 
!
CALL FileOutput
!
WRITE(*,*)
WRITE (*,*) "Process Another Batch File Y/N"
WRITE (*, *)
			READ (*,*) Answer 
		IF (Answer=="Y" .OR. Answer=="y") THEN
	       DEALLOCATE(OutputTable, DataDate)
		   WRITE (*,*)
		   GOTO 1
		END IF
!
! *************************************************************************
!	    
									CONTAINS
!
! *************************************************************************
! 
! Count Lines in BatchFileIn
SUBROUTINE CountFiles ! ****************************************************
IMPLICIT NONE
!
OPEN (UNIT=11,FILE=BatchFileName, STATUS="OLD", POSITION="REWIND", &
         IOSTAT=OpenStatus1)
!
BatchFileNumber=0
DO J=1, 200 
READ (UNIT=11, FMT=10,IOSTAT=InputStatus1) Temp
	IF (InputStatus1 > 0) STOP "Input Error"
			IF (InputStatus1 < 0) EXIT !"End of File"
			BatchFileNumber=BatchFileNumber+1
END DO
10 FORMAT(A)
!
CLOSE (UNIT=11)
!
END SUBROUTINE CountFiles ! ************************************************
!
SUBROUTINE SetUpOutputTable ! **********************************************
IMPLICIT NONE
!
!Find Julian Day of First and Last
 CALL JulDayFormat (First, Month, Dayi, Yeari)
 JulianDay1=julday(Month,Dayi,Yeari)
 CALL JulDayFormat (Last, Month, Dayi, Yeari) 
 JulianDay2=julday(Month,Dayi,Yeari)
!
ALLOCATE (DataDate(JulianDay1:JulianDay2))
!
ALLOCATE (OutputTable(JulianDay1:JulianDay2, 1:BatchFileNumber))
			OutputTable(JulianDay1:JulianDay2, 1:BatchFileNumber)=-99
	DO I=JulianDay1, JulianDay2 ! Array of Calendar Dates
	      JulianDay=I
		  CALL Calendar_Date
		  DataDate(I)=DateTemp(1:10)
	END DO
!
END SUBROUTINE SetUpOutputTable ! ******************************************
!
!
SUBROUTINE ReadBatchfile ! *************************************************
IMPLICIT NONE
!
OPEN (UNIT=11,FILE=BatchFileName, STATUS="OLD", POSITION="REWIND", &
         IOSTAT=OpenStatus1)
!
Batch_Input_Loop:DO J=1, BatchFileNumber
!
     READ (UNIT=11, FMT=10,IOSTAT=InputStatus1) Temp !BatchFileIn
	        BatchFileIn=Temp(1:25)
		IF (InputStatus1 > 0) STOP "Input Error"
			IF (InputStatus1 < 0) EXIT !"End of File"
	10 FORMAT (A)
	 FileNameIn(J)=TRIM(DirectoryIn)//"\"//BatchFileIn
!
END DO Batch_Input_Loop
!
CLOSE (UNIT=11)
!
END SUBROUTINE ReadBatchfile ! ********************************************** 
!
!
! Read Input Files and Populate Output Table 
SUBROUTINE ReadInputFile ! **************************************************
IMPLICIT NONE
!
DO J=1, BatchfileNumber
	NumberLines=0
OPEN (UNIT=10,FILE=FileNameIn(J), STATUS="OLD", ACTION="READ",POSITION="REWIND", &
IOSTAT=OpenStatus)
	IF (OpenStatus > 0) STOP "Cannot Open File (1)"                
                 DO I=1, 54788 ! Count Lines in File and Find First and Last Date
                    READ (UNIT=10,FMT=10, IOSTAT=InputStatus) Temp
					10 FORMAT (A26)					
				   	   IF (InputStatus < 0) EXIT !End of File
					   IF (I==1) First=Temp(1:10)
							NumberLines=NumberLines+1
								    Last=Temp(1:10)									
					IF (Temp(14:26)=="T") Temp(14:26)="0" !Change T(race) to 0
							! Read Data as Real
							READ (Temp(12:26), *) DayValueTemp
							DayValue(I)=DayValueTemp											
                 END DO
	! Find Julian Day of First and Last
		CALL JulDayFormat (First, Month, Dayi, Yeari)
		JulianDay1=julday(Month,Dayi,Yeari)
		CALL JulDayFormat (Last, Month, Dayi, Yeari) 
		JulianDay2=julday(Month,Dayi,Yeari)
!
				OutputTable(JulianDay1:JulianDay2,J)=DayValue(1:NumberLines)	
!			  			
END DO
!
CLOSE (UNIT=10)
!
END SUBROUTINE ReadInputFile ! **********************************************
!
!
SUBROUTINE FileOutput
IMPLICIT NONE
!
OPEN (UNIT=10,FILE=FileNameOut, STATUS="NEW", ACTION="WRITE",POSITION="REWIND", &
IOSTAT=OpenStatus)
	IF (OpenStatus > 0) STOP "Cannot Open File (2)"
!
OPEN (UNIT=11,FILE=BatchFileName, STATUS="OLD", POSITION="REWIND", &
         IOSTAT=OpenStatus1)
!
! Write Column Headers
DO I=1, BatchFileNumber
READ (UNIT=11, FMT=200, IOSTAT=InputStatus1) Temp !BatchFileIn	        
     IF (I==1) WRITE (UNIT=10,FMT=190, ADVANCE="NO") "Date"
		PositionTxt=INDEX(Temp(1:25), ".txt") ! Remove .txt Extension
	 IF (PositionTxt==0) WRITE(UNIT=10, FMT=200, ADVANCE="NO") TRIM(Temp(1:25))
	 IF (PositionTxt>0) WRITE(UNIT=10, FMT=200, ADVANCE="NO") TRIM(Temp(1:PositionTxt-1))	 
        IF (InputStatus1 < 0) EXIT !"End of File"  
END DO
!
190 FORMAT(A4, X)
200 FORMAT (A, X)
300 FORMAT(X, F12.4)
310 FORMAT(A10,X)
WRITE(UNIT=10, FMT=300)
!
! Write Data Columns
DO I=FirstJulianDay, LastJulianDay
	DO J=1,BatchfileNumber
	    IF (J==1) WRITE (UNIT=10, FMT=310, ADVANCE="NO") DataDate(I)  
		WRITE (UNIT=10, FMT=300, ADVANCE="NO") OutputTable (I,J)
	END DO
	     WRITE(UNIT=10, FMT=300)
END DO
!
CLOSE (UNIT=10)
CLOSE (UNIT=11)
!
END SUBROUTINE FileOutput ! *******************************************************



! ************************* Date Subroutines and Functions ******************
!
SUBROUTINE JulDayFormat(Date_Str, Mth, Day, Year) ! *************************
IMPLICIT NONE
! 
CHARACTER, INTENT (IN) :: Date_Str*10
INTEGER, INTENT (OUT) :: Mth, Day, Year 
	READ (Date_Str(1:2),*) Mth
	READ (Date_Str(4:5),*) Day
	READ (Date_Str(7:10),*) Year
! 
END SUBROUTINE JulDayFormat ! ***********************************************
!
!Julian Day from Calendar Date.  Code from Press and others (1992, p.13).
!Modifications suggested by Ahlquist (1999) were made 1/13/2005
! 
FUNCTION julday(mm,id,iyyy) ! ***********************************************
IMPLICIT NONE
      INTEGER julday,id,iyyy,mm,IGREG
      PARAMETER (IGREG=15+31*(10+12*1582))
      INTEGER ja,jm,jy
      jy=iyyy
      if (jy.eq.0) pause 'julday: there is no year zero'
      if (jy.lt.0) jy=jy+1
      if (mm.gt.2) then
        jm=mm+1
      else
        jy=jy-1
        jm=mm+13
      endif
      julday=int(365.25*jy)+int(30.6001*jm)+id+1720995
      if (id+31*(mm+12*iyyy).ge.IGREG) then
        ja=jy/100 !ja=int(0.01*jy) Original line
        julday=julday+2-ja+int(0.25*ja)
      endif

END FUNCTION julday !  **********************************************
!
!
SUBROUTINE Calendar_Date
!
	CALL caldat (JulianDay, Mth, Day, Year)
		WRITE (Month_str,'(I2)') Mth
			IF (Mth<10) Month_str(1:1)="0"
		WRITE (Day_str,'(I2)') Day
			IF (Day<10) Day_str(1:1)="0"
		WRITE (Year_str,'(I4)') Year
			DateTemp=Month_str//"/"// Day_str//"/"//Year_str	    
!
END SUBROUTINE Calendar_Date
!
!
SUBROUTINE caldat(julian,mm,id,iyyy)
      INTEGER id,iyyy,julian,mm,IGREG
      PARAMETER (IGREG=2299161)
      INTEGER ja,jalpha,jb,jc,jd,je
      if(julian.ge.IGREG)then
        jalpha=int(((julian-1867216)-0.25)/36524.25)
        ja=julian+1+jalpha-int(0.25*jalpha)
      else
        ja=julian
      endif
      jb=ja+1524
      jc=int(6680.+((jb-2439870)-122.1)/365.25)
      jd=365*jc+int(0.25*jc)
      je=int((jb-jd)/30.6001)
      id=jb-jd-int(30.6001*je)
      mm=je-1
      if(mm.gt.12)mm=mm-12
      iyyy=jc-4715
      if(mm.gt.2)iyyy=iyyy-1
      if(iyyy.le.0)iyyy=iyyy-1
END SUBROUTINE caldat 

END PROGRAM SetUpDailyTable
